import wx
from random import *
from math import *
from pickle import *
class PokerLabel(wx.StaticBitmap):
	def __init__(self, parent=None, img=None):
		wx.StaticBitmap.__init__(self, parent, -1, img.ConvertToBitmap())
		
		self.Bind( wx.EVT_SIZE, self.onSize )
		
		self.image = img
		
		self.size = img.GetSize()
		
	def setBg(self, img):
		self.image = img
		img = self.image.Scale(self.size.width,self.size.height)
		self.SetBitmap(img.ConvertToBitmap())
		
	def onSize(self, evt):
		if evt.GetSize().width==0 or evt.GetSize().height==0:
			return
		self.size = evt.GetSize()
		img = self.image.Scale(self.size.width,self.size.height)
		self.SetBitmap(img.ConvertToBitmap())
		pass
		
class PokerFrame(wx.Frame):
	def __init__(self, parent=None, id=-1, title='poker'):
		wx.Frame.__init__(self, parent, id, title)
		
		#local data
		self.rows = 4
		self.cols = 9
		self.curEmptyLabel = 0
		self.running = False
		self.indexOfUseful = 0
		self.label = []
		self.usefulIndex = [i for i in range(0,52)]
		self.usedIndex = []
		
		#image data
		self.ttpic = []
		for i in range(0,54):
			self.ttpic.append( wx.Image( 'img/%d.jpg'%(i+1), wx.BITMAP_TYPE_JPEG ) )
		self.picNone = wx.Image( 'img/none.jpg', wx.BITMAP_TYPE_JPEG )
		self.picLogo = wx.Image( 'img/logo.jpg', wx.BITMAP_TYPE_JPEG )
		
		#widget
		self.logo = wx.StaticBitmap(self, -1, self.picLogo.ConvertToBitmap())
		self.btn = wx.BitmapButton(self, -1, self.ttpic[52].ConvertToBitmap())
		self.undo = wx.BitmapButton(self, -1, wx.Image( 'img/undo.jpg', wx.BITMAP_TYPE_JPEG ).ConvertToBitmap(), style=wx.NO_BORDER|wx.BU_LEFT )
		self.poker = wx.Panel(self, -1)
		self.poker.SetSizer( wx.GridSizer(1,1,2,2) )
		#sizer
		box_v = wx.BoxSizer(wx.VERTICAL)
		box_v.Add(self.logo, 0, wx.EXPAND)
		box_v.Add((0,0), 1)
		box_v.Add(self.btn, 0, wx.ALIGN_CENTER)
		box_v.Add((0,0), 1)
		box_v.Add(self.undo, 0, wx.EXPAND)
		
		box_h = wx.BoxSizer(wx.HORIZONTAL)#VERTICAL
		box_h.Add(box_v, 0, wx.EXPAND)
		box_h.Add(self.poker, 1, wx.EXPAND)
		
		self.SetSizer(box_h)
		
		#other
		self.Bind(wx.EVT_BUTTON, self.onBtnChange, self.btn)
		self.Bind(wx.EVT_TIMER, self.onTimer)
		self.Bind(wx.EVT_BUTTON, self.onBtnUndo, self.undo)
		
		self.timer = wx.Timer(self)
		
		#start
		self.resizePokerLabel(4, 9)
		
			
	def onTimer(self, evt):
		i = int(uniform(0,len(self.usefulIndex)))
		self.indexOfUseful = i
		
		tIndex = self.usefulIndex[ self.indexOfUseful ]
		try:
			self.btn.SetBitmap(self.ttpic[tIndex].ConvertToBitmap())
		except Exception:
			print tIndex
		
	def onBtnChange(self, evt):
		self.running = not self.running
		if not self.running:
			self.timer.Stop()
			if self.curEmptyLabel<self.rows*self.cols:
				self.btn.SetBitmap(self.ttpic[52].ConvertToBitmap())
				tIndex = self.usefulIndex[ self.indexOfUseful ]
				self.label[self.curEmptyLabel].setBg( self.ttpic[tIndex] )
				self.usedIndex.append( self.usefulIndex[self.indexOfUseful] )
				del self.usefulIndex[self.indexOfUseful]
				self.curEmptyLabel = self.curEmptyLabel + 1
		else:
			if self.curEmptyLabel<self.rows*self.cols:
				self.timer.Start(10)
			
	def onBtnUndo(self, evt):
		if self.curEmptyLabel!=0:
			self.curEmptyLabel = self.curEmptyLabel-1
			self.label[self.curEmptyLabel].setBg(self.picNone)
			self.indexOfUseful = self.usedIndex[ len(self.usedIndex)-1 ]
			self.usefulIndex.append( self.usedIndex[len(self.usedIndex)-1] )
			del self.usedIndex[-1:]
			
	def resizePokerLabel(self, rows, cols):
		self.setStatus( [rows,cols,0,False,0,[i for i in range(52)],[]] )
		
	def getStatus(self):
		return (self.rows,
				self.cols,
				self.curEmptyLabel,
				self.running, 
				self.indexOfUseful,
				self.usefulIndex,
				self.usedIndex
				)
		pass
	def setStatus(self, st):
		self.rows = st[0]
		self.cols = st[1]
		self.curEmptyLabel = st[2]
		self.running = st[3]
		self.indexOfUseful = st[4]
		self.usefulIndex = st[5]
		self.usedIndex = st[6]
		#print self.usedIndex
		box = self.poker.GetSizer()
		box.Clear(True)
		#print box.GetRows(), box.GetCols()
		#print box.GetVGap(), box.GetHGap()
		self.label = []
		box.SetRows(self.rows)
		box.SetCols(self.cols)
		#print self.rows, self.cols
		for i in range(0, self.rows*self.cols):
			t = PokerLabel(self.poker, self.picNone)
			box.Add( t, 0, wx.EXPAND )
			self.label.append( t )
		for i in range(0, len(self.usedIndex)):
			self.label[i].setBg( self.ttpic[self.usedIndex[i]] )
			
		box.Layout()
		
class GetPokerSize(wx.Dialog):
	def __init__(self, parent):
		pre = wx.PreDialog()
		pre.SetExtraStyle(wx.DIALOG_EX_CONTEXTHELP)
		pre.Create(parent, -1, '', wx.DefaultPosition, wx.DefaultSize, style=wx.DEFAULT_DIALOG_STYLE)
		self.PostCreate(pre)
		
		rt = wx.StaticText(self, -1, 'the rows of poker:')
		self.re = wx.TextCtrl(self, -1, str(f.rows))
		
		ct = wx.StaticText(self, -1, 'the cols of poker:')
		self.ce = wx.TextCtrl(self, -1, str(f.cols))
		
		ok = wx.Button(self, -1, 'ok')
		cancel = wx.Button(self, wx.ID_CANCEL, 'cancel')
		
		g = wx.GridSizer(3,2,2,2)
		g.Add(rt, 0)
		g.Add(self.re, 0)
		g.Add(ct, 0)
		g.Add(self.ce, 0)
		g.Add(ok, 0)
		g.Add(cancel, 0)
		self.SetSizer(g)
		g.Fit(self)
	
		self.Bind(wx.EVT_BUTTON, self.onononOk, ok)
	
	def onononOk(self, evt):
		global f
		r = int( self.re.GetValue() )
		c = int( self.ce.GetValue() )
		if r*c>52:
			dt = wx.MessageDialog(self, 'please make row*col<=52', 'error')
			dt.ShowModal()
			dt.Destroy()
			return
		f.resizePokerLabel(r, c)
		self.Destroy()
		
class Manager(wx.Dialog):
	def __init__(self, parent):
		pre = wx.PreDialog()
		pre.SetExtraStyle(wx.DIALOG_EX_CONTEXTHELP)
		pre.Create(parent, -1, '', wx.DefaultPosition, wx.DefaultSize, style=wx.DEFAULT_DIALOG_STYLE)
		self.PostCreate(pre)
		
		#ok
		rt = wx.StaticText(self, -1, 'name:')
		self.name = wx.TextCtrl(self, -1, 'UnTitled')
		ok = wx.Button(self, -1, 'save')
		self.Bind(wx.EVT_BUTTON, self.onOk, ok)
		st = wx.BoxSizer(wx.HORIZONTAL)
		st.AddMany([ (rt,0,wx.EXPAND), ((0,0), 0, wx.EXPAND),(self.name,0,wx.EXPAND), (ok,0,wx.EXPAND)])
		#self.SetSizer(s)
		
		#list box
		self.mgr = wx.ListBox(self, -1)
		self.mgr.Fit()
		#apply delete
		apply = wx.Button(self, -1, 'apply')
		delete = wx.Button(self, -1, 'delete')
		sm = wx.BoxSizer(wx.HORIZONTAL)
		sm.AddMany([apply, delete])
		self.Bind(wx.EVT_BUTTON, self.onApply, apply)
		self.Bind(wx.EVT_BUTTON, self.onDelete, delete)
		
		#global
		s = wx.BoxSizer(wx.VERTICAL)
		s.AddMany([ (st,0,wx.EXPAND), (self.mgr,100,wx.EXPAND), (sm,0,wx.EXPAND)])
		self.SetSizer(s)
		self.Fit()
		
		self.load()
		
	def onOk(self, evt):
		self.mgr.Append( self.name.GetValue(), f.getStatus() )
		self.name.Clear()
		self.save()
		
	def onApply(self, evt):
		if -1 != self.mgr.GetSelection():
			f.setStatus( self.mgr.GetClientData(self.mgr.GetSelection()) )

	def onDelete(self, evt):
		if -1 != self.mgr.GetSelection():
			self.mgr.Delete(self.mgr.GetSelection())
		pass
		self.save()
		
	def save(self):
		try:
			file = open('saves.dat', 'wb')
			for i in range(0, self.mgr.GetCount()):
				name = self.mgr.GetString(i)
				data = self.mgr.GetClientData(i)
				dump( (name, data), file )
		except Exception as ee:
			wx.MessageDialog.Show( str(ee), 'error' )
			pass
	def load(self):
		self.mgr.Clear()
		try:
			file = open('saves.dat', 'rb')
		except Exception:
			file = open('saves.dat', 'wb')
			file = open('saves.dat', 'rb')
			
		try:
			while True:
				(name, data) = load(file)
				self.mgr.Append( name, data )
		except Exception:
			pass
		
# __main__	
app = wx.App(False)

f = PokerFrame()

#menu
def onMenuPokerSize(evt):
	GetPokerSize(f).ShowModal()
def onMenuManager(evt):
	Manager(f).ShowModal()
	
menubar = wx.MenuBar()
menu = wx.Menu()
menu.Append(100, 'SetPokerSize', '')
menu.Append(101, 'Manager', '')
menubar.Append(menu, '&File')
f.SetMenuBar(menubar)
f.Bind(wx.EVT_MENU, onMenuPokerSize, id=100)
f.Bind(wx.EVT_MENU, onMenuManager, id=101)

f.Show()

app.MainLoop()	